/****************************************************************
 *  (C)  The TriTechnologies Company, 1991                      *
 *  	      	   	     	       	    	      	   	*
 *       This software is furnished under a license and may     *
 *       be used and copied only in accordance with the terms   *
 *       of such license and with the inclusion of the above    *
 *       copyright notice.  this software or any other copies   *
 *       thereof may not be provided or otherwise made avail-   *
 *       able to any other person.  No title to and ownership   *
 *       of the software is hereby transferred.                 *
 *                                                              *
 *       The information in this software is subject to change  *
 *       without notice and should not be construed as a        *
 *       commitment by The TriTechnologies Company.             *
 ****************************************************************
 *
 * Facility:
 *
 *     pcthread.h
 *
 * Abstract:
 *
 *       PCthreads(tm) Application Programming Interface, API.
 *       The functions declared in this header file may be called
 *       by application programs (aka client applications) whose 
 *       design requires multiple threads of execution.  These 
 *       functions provide a complete set of primitives necessary
 *       for the creation, run-time management, and deletion of
 *       threads, mutexes, and condition variables.  Moreover, 
 *       these routines are all that are required to design higher
 *       level abstractions, such as monitors, counting sema-
 *       phores, friendly mutexes, etc,.
 *
 */

#ifndef _PCTHREAD_H
#define _PCTHREAD_H

#include "typedefs.h"

/*
 * These typedefs are required by the P1003.4a standard and relieve the
 * user from the tedium of allocating attributes handles at runtime when
 * the client code will be using the system's default attributes.
 */

#define pcthread_attr_default       (*(pcthread_attr_getdefault_ns()))

#define pcthread_mutexattr_default  (*(pcthread_mutexattr_getdefault_ns()))

#define pcthread_condattr_default   (*(pcthread_condattr_getdefault_ns()))







/*
 * Clients may initialize the Run-Time Management System, RTMS using
 * their own settings.  This is not a required service.  For it to have
 * any effect it must be the very first service called.  If called
 * subsequent to any other service, it will have no effect.
 */

int
pcthread_attr_setdefaults_ns( policy_t   scheduling_policy,
                              unsigned   thread_quantum,
                              int        max_thread_priority,
                              int        default_thread_priority,
                              unsigned   default_thread_stack_size,
                              boolean_t  thread_inheritance );


/*
 * The following services allow client threads to operate on thread
 * attributes objects.
 */
 
int
pcthread_attr_create( pcthread_attr_t * );

int
pcthread_attr_delete( pcthread_attr_t * );

int
pcthread_attr_setprio( pcthread_attr_t *attr, int );

int
pcthread_attr_getprio( pcthread_attr_t );

int
pcthread_attr_setsched( pcthread_attr_t *attr, policy_t policy );

policy_t
pcthread_attr_getsched( pcthread_attr_t attr );

int
pcthread_attr_setinheritsched( pcthread_attr_t *attr, boolean_t inherit );

int
pcthread_attr_getinheritsched( pcthread_attr_t attr );

int
pcthread_attr_setstacksize( pcthread_attr_t *, unsigned );

int
pcthread_attr_getstacksize( pcthread_attr_t );

int
pcthread_attr_getmaxprio_ns( void );

int
pcthread_attr_setquantum_ns( pcthread_attr_t *attr, unsigned quantum );

int
pcthread_attr_getquantum_ns( pcthread_attr_t attr );

pcthread_attr_t *
pcthread_attr_getdefault_ns( void );

int
pcthread_attr_print_ns( pcthread_attr_t );

/*
 * These functions allow client threads to operate on mutex attributes
 * objects.
 */

int
pcthread_mutexattr_create( pcthread_mutexattr_t *attr );

int
pcthread_mutexattr_delete( pcthread_mutexattr_t *attr );

int
pcthread_mutexattr_setprotocol( pcthread_mutexattr_t *attr, 
                                pcthread_protocol_t protocol );
pcthread_protocol_t
pcthread_mutexattr_getprotocol( pcthread_mutexattr_t attr );

int
pcthread_mutexattr_setprio_ceiling( pcthread_mutexattr_t *attr, 
                                    int prio_ceiling );

int
pcthread_mutexattr_getprio_ceiling( pcthread_mutexattr_t attr );

pcthread_mutexattr_t *
pcthread_mutexattr_getdefault_ns( void );

int
pcthread_mutexattr_print_ns( pcthread_mutexattr_t );

/*
 * These services allow client threads to operate on condition variable
 * attributes objects.
 */

int
pcthread_condattr_create( pcthread_condattr_t *attr );

int
pcthread_condattr_delete( pcthread_condattr_t *attr );

pcthread_condattr_t *
pcthread_condattr_getdefault_ns( void );

int
pcthread_condattr_print_ns( pcthread_condattr_t );








/*  
 *  Thread-specific services
 */
 
int
pcthread_create(   pcthread_t *,            /* A handle to a thread */
                   pcthread_attr_t,         /* Thread Attributes handle */
                   pcthread_action_t,       /* The thread's action routine */
                   pcthread_addr_t *);      /* Arg to the action routine */

/*  
 *  A thread may wait for the completion of another and obtain the return
 *  value of its action routine, if any.
 */

int
pcthread_join( pcthread_t *, pcthread_addr_t *);

/*
 * A thread may terminate itself and pass a status value back to any thread
 * that is waiting for it to complete (i.e., via the pcthread_join()
 * service ).
 */
 
void
pcthread_exit( pcthread_addr_t status );

/*
 * A thread may use pcthread_detach() to indicate to the run time system
 * that its resources (e.g., memory) may be reclaimed upon termination.
 * This does not cause the thread to terminate, but does preclude any
 * threads from joining with it, once pcthread_detach() has been called.
 */

int
pcthread_detach( pcthread_t *th_handle );

/*  
 *  Client applications may adjust a thread's priority    
 */

int
pcthread_setprio( pcthread_t *th_handle, unsigned priority );

int
pcthread_getprio( pcthread_t *th_handle );

int
pcthread_setactive_prio_ns( unsigned );

int
pcthread_getactive_prio_ns( void );

int
pcthread_setscheduler( pcthread_t *, policy_t );  /* Not yet implemented */

policy_t
pcthread_getscheduler( pcthread_t * );           /* Not yet implemented */


/* 
 *  Client threads may obtain their own handles
 */

pcthread_t *
pcthread_self( void );

/* 
 *  Client threads may unconditionally yield the processor.
 */

void
pcthread_yield( void );

/* 
 *  Client threads may execute a timed delay for N milliseconds 
 */

void
pcthread_delay_ns( unsigned );

/*
 * Client applications may test two threads for equality
 */
 
boolean_t
pcthread_equal( pcthread_t, pcthread_t );

/*
 * Client threads may operate on mutexes
 */

int
pcthread_mutex_init( pcthread_mutex_t *, pcthread_mutexattr_t );

int
pcthread_mutex_destroy( pcthread_mutex_t * );

int
pcthread_mutex_lock( pcthread_mutex_t *);

int
pcthread_mutex_unlock( pcthread_mutex_t *);

int
pcthread_mutex_trylock( pcthread_mutex_t * );

int
pcthread_mutex_printdata_ns( pcthread_mutex_t );

/*
 * Client threads may operate on condition variables
 */

int
pcthread_cond_init( pcthread_cond_t *, pcthread_condattr_t );

int
pcthread_cond_destroy( pcthread_cond_t * );

int
pcthread_cond_wait( pcthread_cond_t *, pcthread_mutex_t *);

int
pcthread_cond_timed_wait( pcthread_cond_t *, pcthread_mutex_t *, unsigned );

int
pcthread_cond_signal( pcthread_cond_t * );

int
pcthread_cond_broadcast( pcthread_cond_t *);

void
pcthread_cond_printdata_ns( pcthread_cond_t );

/* 
 * Threads need to disable and reenable preemption to protect non-reentrant
 * routines -- E.g., I/O routines, malloc and other lib calls, etc.
 */

int
pcthread_disable_preemption_ns( void );

void
pcthread_enable_preemption_ns( int );

/*
 * These two services cause the system to keep track of every thread state
 * change.  A record is logged if, and only if, the state change results in
 * the rescheduling of the thread undergoing the state change.
 */

void
pcthread_enable_trace_ns( void );

void
pcthread_disable_trace_ns( void );

/*
 * Shut down the system gracefully.  Client threads should use this service
 * when, for example, they detect an unrecoverable error.
 */
 
void
pcthread_close_ns( char *module_name,
                   char *function_name,
                   char *exception_message,
                   exception_t exception_code );


unsigned
pcthread_checkstack_ns( char *function_name );

#endif
